#!/usr/bin/python3

from PyQt5.QtGui import *
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
import psycopg2, sys

myhost = 'localhost' #'scischool.in'
mydb = 'cuqbank'

MAXMOD = 8

class MainWindow(QMainWindow):   # Creating main window class
	dbcursor = None
	dbpasswd = None
	courses = None
	courseCode = None
	subFilter = 'All'
	semFilter = 'All'
	subject = None
	degree = None
	syllYear = None
	semester = None
	language = None
	qpModel = None
	duration = None
	numModules = None
	moduleMarkString = None
	subjects = ['Mathematics', 'EVS', 'Statistics', 'Physics', 'Philosophy', 'Hindi', 'Malayalam','Arabic','English']
	subjectFilter = ['All'] + subjects
	#,'Mathematics', 'EVS', 'Statistics', 'Physics', 'Philosophy', 'Hindi', 'Malayalam','Arabic']
	degrees = ['BA', 'BSc', 'MA', 'MSc']
	syllYears = ['2020', '2021', '2022', '2023', '2024', '2025']
	semesters = ['First', 'Second', 'Third', 'Fourth', 'Fifth', 'Sixth', 'Seventh', 'Eighth']
	semesterFilter = ['All']+semesters
	#,'First', 'Second', 'Third', 'Fourth', 'Fifth', 'Sixth', 'Seventh', 'Eighth']
	languages = ['english', 'hindi', 'malayalam','arabic']
	qpModels = ['M50CU','M70CU','M50KU','M60KU','M70KU','M48KU','M80KU']
	durations = ['90 min', '120 min', '180 min']
	qpMarks = {'M50CU':50,'M70CU':70,'M50KU':50,'M60KU':60,'M70KU':70,'M48KU':48,'M80KU':80}
	 
	def __init__(self, *args, **kwargs):
		super(MainWindow, self).__init__(*args, **kwargs)
		
		self.setGeometry(100, 100, 1200, 600)			
		fixedfont = QFontDatabase.systemFont(QFontDatabase.FixedFont)
		fixedfont.setPointSize(10)

		layout = QVBoxLayout()	
		vbox1 = QVBoxLayout()
		layout.addLayout(vbox1)
	
		hbox1 = QHBoxLayout()
		vbox1.addLayout(hbox1)
		x = QLabel('Server')
		hbox1.addWidget(x)
		self.hostLineEdit = QLineEdit(myhost)
		self.hostLineEdit.setMaximumWidth(200)
		hbox1.addWidget(self.hostLineEdit)

		self.dbConnectButton = QPushButton('Connect to DB->')
		hbox1.addWidget(self.dbConnectButton)
		self.dbConnectButton.clicked.connect(self.updateCourseList)
		x = QLabel('Database Name ')
		hbox1.addWidget(x)		
		self.dbLineEdit = QLineEdit(mydb)
		self.dbLineEdit.setMaximumWidth(200)
		hbox1.addWidget(self.dbLineEdit)

		self.filterSubjectButton = QPushButton('Subject: All')
		hbox1.addWidget(self.filterSubjectButton)
		m = QMenu()
		for sub in self.subjectFilter:
			m.addAction(sub, lambda item=sub: self.setSubjectFilter(item))
		self.filterSubjectButton.setMenu(m)
		
		self.filterSemesterButton = QPushButton('Semester: All')
		hbox1.addWidget(self.filterSemesterButton)
		m = QMenu()
		for sub in self.semesterFilter:
			m.addAction(sub, lambda item=sub: self.setSemesterFilter(item))
		self.filterSemesterButton.setMenu(m)

		hbox1.addStretch()

		hb = QHBoxLayout()
		vbox1.addLayout(hb)
		self.qeditor = QPlainTextEdit()
		self.qeditor.setFont(fixedfont)
		self.qeditor.setLineWrapMode(1)
		hb.addWidget(self.qeditor)

		hb = QHBoxLayout()
		vbox1.addLayout(hb)
		x = QLabel('To Create Provide all details. To Delete Couse Code is enough')
		hb.addWidget(x)

		hbox4 = QHBoxLayout()
		vbox1.addLayout(hbox4)	
		self.subjectButton = QPushButton('Subject')
		hbox4.addWidget(self.subjectButton)
		m = QMenu()
		for sub in self.subjects:
			m.addAction(sub, lambda item=sub: self.selectSubject(item))
		self.subjectButton.setMenu(m)
		
		self.degreeButton = QPushButton('Degree')
		hbox4.addWidget(self.degreeButton)
		m = QMenu()
		for deg in self.degrees:
			m.addAction(deg, lambda item=deg: self.selectDegree(item))
		self.degreeButton.setMenu(m)

		self.syllYearButton = QPushButton('Syllabus Year')
		hbox4.addWidget(self.syllYearButton)
		m = QMenu()
		for s in self.syllYears:
			m.addAction(s, lambda item=s: self.selectYear(item))
		self.syllYearButton.setMenu(m)

		self.semesterButton = QPushButton('Semester')
		hbox4.addWidget(self.semesterButton)
		m = QMenu()
		for s in self.semesters:
			m.addAction(s, lambda item=s: self.selectSemester(item))
		self.semesterButton.setMenu(m)

		self.languageButton = QPushButton('Language')
		hbox4.addWidget(self.languageButton)
		m = QMenu()
		for s in self.languages:
			m.addAction(s, lambda item=s: self.selectLanguage(item))
		self.languageButton.setMenu(m)
		
		self.qpModelButton = QPushButton('QP Model')
		hbox4.addWidget(self.qpModelButton)
		m = QMenu()
		for s in self.qpModels:
			m.addAction(s, lambda item=s: self.selectQpModel(item))
		self.qpModelButton.setMenu(m)
		
		self.durationButton = QPushButton('Duration')
		hbox4.addWidget(self.durationButton)
		m = QMenu()
		for s in self.durations:
			m.addAction(s, lambda item=s: self.selectDuration(item))
		self.durationButton.setMenu(m)

		hbox5 = QHBoxLayout()
		vbox1.addLayout(hbox5)
		spacer = QLabel(' '*34)
		hbox5.addWidget(spacer)
		self.numModulesButton = QPushButton('numModules')
		self.numModulesButton.setVisible(False)
		hbox5.addWidget(self.numModulesButton)
		m = QMenu()
		for s in  ['4', '5', '6','7', '8']:
			m.addAction(s, lambda item=s: self.selectnumModules(item))
		self.numModulesButton.setMenu(m)

		self.pmLabels = [None]*MAXMOD
		self.pmEdits = [None]*MAXMOD
		for k in range(MAXMOD):
			self.pmLabels[k] = QLabel('M%d Mark->'%(k+1))
			self.pmLabels[k].setVisible(False)
			hbox5.addWidget(self.pmLabels[k])
			self.pmEdits[k] = QLineEdit('0')
			self.pmEdits[k].setMaximumWidth(50)
			self.pmEdits[k].setVisible(False)
			hbox5.addWidget(self.pmEdits[k])
		hbox5.addStretch()

		x = QLabel('To Add a Course provide all the information. To remove,  courseCode is enough.')
		#hbox2.addWidget(x)
		
		hbox3 = QHBoxLayout()
		layout.addLayout(hbox3)

		f = QFormLayout()
		vbox1.addLayout(f)
		self.codeLineEdit = QLineEdit('')
		f.addRow(QLabel("Course Code"), self.codeLineEdit)

		self.pass1LineEdit = QLineEdit('')
		self.pass1LineEdit.setEchoMode(QLineEdit.Password)
		f.addRow(QLabel("Course password"), self.pass1LineEdit)

		self.pass2LineEdit = QLineEdit('')
		self.pass2LineEdit.setEchoMode(QLineEdit.Password)
		f.addRow(QLabel("Course password"), self.pass2LineEdit)

		self.nameLineEdit = QLineEdit('')
		f.addRow(QLabel("Course Name"), self.nameLineEdit)

		hbox5 = QHBoxLayout()
		vbox1.addLayout(hbox5)
		self.courseButton = QPushButton('Select Existing Course')
		self.courseButton.clicked.connect(self.selectCourse)
		hbox5.addWidget(self.courseButton)
		self.courseMenu = QMenu()
		self.courseButton.setMenu(self.courseMenu)
		self.delButton = QPushButton('Delete Selected Course')
		self.delButton.clicked.connect(self.delCourse)
		hbox5.addWidget(self.delButton)
		self.addButton = QPushButton('Create New Course')
		self.addButton.clicked.connect(self.addCourse)
		hbox5.addWidget(self.addButton)
		hbox5.addStretch()

		hb = QHBoxLayout()
		layout.addLayout(hb)	
		self.msg = QLabel('')
		hb.addWidget(self.msg)
		
		container = QWidget()
		container.setLayout(layout)
		self.setCentralWidget(container)  	# making it central widget	
		self.show()		# End __init__
		
	def setSubjectFilter(self,s):
		self.subFilter = s
		self.filterSubjectButton.setText('Subject:'+s)
		self.displayCourses()

	def setSemesterFilter(self,s):
		self.semFilter = s
		self.filterSemesterButton.setText('Semester:'+s)
		self.displayCourses()

	def selectSubject(self,s):
		self.subject = s
		self.subjectButton.setText(s)
				
	def selectDegree(self,s):
		self.degree = s
		self.degreeButton.setText(s)			

	def selectYear(self,s):
		self.syllYear = s
		self.syllYearButton.setText(s)

	def selectSemester(self,s):
		self.semester = s
		self.semesterButton.setText(s)

	def selectLanguage(self,s):
		self.language = s
		self.languageButton.setText(s)

	def selectQpModel(self,s):
		self.qpModel = s
		self.qpModelButton.setText(s)
		self.numModulesButton.setVisible(True)
		self.numModulesButton.setText('numModules')
		self.numModules = None		# Reset number of modules
		for k in range(MAXMOD):
			self.pmLabels[k].setVisible(False)
			self.pmEdits[k].setVisible(False)

	def selectDuration(self,s):
		self.duration = s
		self.durationButton.setText(s)

	def selectnumModules(self,s):		# Also controls the mark widgets
		self.numModulesButton.setText(s+' Modules')
		self.numModules = eval(s)
		if self.qpModel not in ['M50CU', 'M70CU']:
			return
		for k in range(MAXMOD):			# Show the mark entry widgets
			if(k < self.numModules):
				self.pmLabels[k].setVisible(True)
				self.pmEdits[k].setVisible(True)
			else:
				self.pmLabels[k].setVisible(False)
				self.pmEdits[k].setVisible(False)
		#self.getModuleMarks()

	def getModuleMarks(self):			# make the string from the Widget inputs
		if self.qpModel == None:
			self.dialog_critical('Error: QP Model is NOT selected')
			return 
		if self.qpModel not in ['M50CU', 'M70CU']:
			self.moduleMarkString = '%d'%self.numModules
			for k in range(self.numModules):
				self.moduleMarkString += ':0'
			print(self.moduleMarkString)
			return True
		ss = '%d'%self.numModules
		tm = 0
		try:
			for k in range(self.numModules):
				ss += ':%s'%self.pmEdits[k].text()
				tm += eval(self.pmEdits[k].text())
		except:
			self.dialog_critical('Invalid marks given.')
			return
		n = self.qpMarks[self.qpModel]	
		if tm != n:
			self.dialog_critical('Section Marks total should be %d'%n)
			return	
		self.moduleMarkString = ss
		return True

	def initModuleMarks(self,ss):
			if self.qpModel not in ['M50', 'M70','M50CU', 'M70CU']:
				return
			sss = ss.split(':')	
			self.numModules = eval(sss[0])
			marks = sss[1:]
			if len(marks) != self.numModules:
				self.dialog_critical('numModules & marks NOT proper. %s'%ss)
				return
			try:
				for k in range(self.numModules):
					self.pmEdits[k].setText(marks[k])
			except:
				self.dialog_critical('Section marks NOT proper.')
				return
			return


	def updateCourseList(self):
		host = self.hostLineEdit.text()
		db = self.dbLineEdit.text()
		try:
			conn = psycopg2.connect(database=db, host=host, user='dummy', password='dum123')
			cursor = conn.cursor()
			sql =  'select * from coursedetails;'
			cursor.execute(sql)
			result = cursor.fetchall()
		except:
			s = 'List Courses: Connection to Database %s failed.'%db
			self.dialog_critical(s)		
			return				

		self.courses = result		# Store the course list information
		self.msg.setText('%d Courses in Database %s.'%(len(result),db))	
		self.displayCourses()
		
		
	def displayCourses(self):
		txt = ''
		self.qeditor.setPlainText(txt)		
		txt = '%-20s %-30s %-15s %-8s %-8s %-8s %-10s %-10s %-10s %-10s\n'%('Course Code','Course Name','Subject','Degree','Year','Semester','Language','QPModel', 'Duration', 'Modules')

		self.courseMenu.clear()
		if self.courses == None:
			return
		for row in self.courses:
			if self.subFilter != 'All' and self.subFilter != row[2]:  	
				continue
			if self.semFilter != 'All' and self.semFilter != row[5]:  	
				continue
			self.courseMenu.addAction(row[0][6:], lambda item=row[0]: self.selectCourse(item))
			
			s = '%-20s %-30s %-15s %-8s %-8s %-8s %-10s %-10s %-10s %-15s\n'%(row[0][6:],row[1],row[2],row[3],row[4],row[5],row[6],row[7],row[8],row[9])
			txt += s[:-1] + '\n'  
		self.qeditor.setPlainText(txt)
		self.qeditor.setReadOnly(True)
		
	def selectCourse(self, code):
		for c in self.courses:
			if code == c[0]:
				print (code)	
				self.codeLineEdit.setText(c[0][6:])
				self.pass1LineEdit.setText('')
				self.pass2LineEdit.setText('')
				self.nameLineEdit.setText(c[1])	
				self.subject = c[2]
				self.subjectButton.setText(self.subject)	
				self.degree = c[3]
				self.degreeButton.setText(self.degree)
				self.syllYear = c[4]
				self.syllYearButton.setText(self.syllYear)
				self.semester = c[5]
				self.semesterButton.setText(self.semester)
				self.language = c[6]
				self.languageButton.setText(self.language)
				self.qpModel = c[7]
				self.qpModelButton.setText(self.qpModel)
				self.selectQpModel(self.qpModel)
				self.duration = c[8]
				self.durationButton.setText(self.duration)	
				np = c[9].split(':')[0]			
				self.numModules = eval(np)
				self.selectnumModules(np)		# control the widgets
				self.numModulesButton.setText('%d Modules'%self.numModules)		
				self.initModuleMarks(c[9])		# Sets the mark LineEdit values
				
	def addCourse(self):	
		if self.courses == None:
			self.dialog_critical('Not Connected to any Database')
			return	
	
		if self.syllYear == None:		# Year to be prefixed with courseCode
			self.dialog_critical('Syllabus Year not selected.')
			return	

		code = self.codeLineEdit.text().strip()
		if len(code) < 6:
			self.dialog_critical('Course Code too short or empty.')
			return	

		code = 'Y' + self.syllYear + '_' +code
		print (code)
		
		for c in self.courses:
			if code == c[0]:
				self.dialog_critical('Existing Course Code | %s'%code)
				return	

		cpw = self.pass1LineEdit.text()
		if len(cpw) < 6:
			self.dialog_critical('Course password must have minimum 6 characters.')
			return	

		if cpw != self.pass2LineEdit.text():
			self.dialog_critical('Course passwords not matching.')
			return	
			
		name = self.nameLineEdit.text()
		if len(name.strip()) < 5:
			self.dialog_critical('Course Name too short or not given.')
			return	
			
		if self.subject == None:
			self.dialog_critical('Subject not selected.')
			return	

		if self.degree == None:
			self.dialog_critical('Degree not selected.')
			return	


		if self.semester == None:
			self.dialog_critical('Semester not selected.')
			return	

		if self.language == None:
			self.dialog_critical('Type setting language not selected.')
			return	

		if self.qpModel == None:
			self.dialog_critical('QP Model not selected.')
			return	

		if self.duration == None:
			self.dialog_critical('Duration not selected.')
			return	

		if self.numModules == None:
			self.dialog_critical('Number of Modules not selected.')
			return	

		if self.getModuleMarks() != True:
			return	

		host = self.hostLineEdit.text()
		db = self.dbLineEdit.text()

		try:
			pw = GUI.getPassword(db)	
			if pw == None:
				self.dialog_critical('No password obtained.')		
				return
			print(host, db, pw)
			conn = psycopg2.connect(database=db, host=host, user=db, password=pw)
			cursor = conn.cursor()
		except:
			self.dialog_critical('Connecting to Database failed.')
			return	
		
		sql = "CREATE TABLE %s(qtext VARCHAR, atext VARCHAR, author VARCHAR, modnum int, qtype CHAR(1), qnum int, relations VARCHAR, mark int, status VARCHAR,figname VARCHAR, figure bytea);"%code
		#print (sql)
		cursor.execute(sql)

		sql = "create user %s with password '%s'"%(code, cpw)
		cursor.execute(sql)
		
		sql = 'grant all on table %s to %s'%(code,code)
		cursor.execute(sql)

		sql = 'grant select on table %s to reader'%(code)
		cursor.execute(sql)

		sql = "insert into coursedetails (code, name, subject, degree, syllabusyear, semester, texlanguage, qpmodel, duration,nmodules) values ('%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s','%s');"\
			%(code, name, self.subject, self.degree, self.syllYear, self.semester, self.language, self.qpModel, self.duration, self.moduleMarkString)
		cursor.execute(sql)
		conn.commit()
		self.updateCourseList()
		
		
	def delCourse(self):
		code = 'Y' + self.syllYear + '_' +self.codeLineEdit.text().strip()
		host = self.hostLineEdit.text()
		db = self.dbLineEdit.text()
		reply = QMessageBox.question(GUI, 'Delete Course', 'Proceed with Delete ?',QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
		if reply == QMessageBox.No:
			return	
		
		try:
			pw = GUI.getPassword(db)	
			if pw == None:
				self.dialog_critical('No password obtained.')		
				return
			conn = psycopg2.connect(database=db, host=host, user=db, password=pw)
			cursor = conn.cursor()
		except:
			self.dialog_critical('Connecting to Database failed.')
			return	

		try:
			sql = "delete from coursedetails where code = '%s'"%code
			cursor.execute(sql)		
			sql = "drop table %s;"%code
			cursor.execute(sql)		
			sql = "drop user %s;"%code
			cursor.execute(sql)
		except:
			self.dialog_critical('Error: Delete Course %s'%code)
		conn.commit()
		self.updateCourseList()
				
	def getPassword(self, cname):
		text, ok = QInputDialog.getText(None, "Accessing Database", "Password for Database '%s'"%cname, 
		QLineEdit.Password)
		if ok and text:
			return text		

	def dialog_critical(self, s):
		dlg = QMessageBox(self)
		dlg.setText(s)
		dlg.setIcon(QMessageBox.Critical)
		dlg.show()			
			

app = QApplication(sys.argv)   # Main program
GUI = MainWindow()
app.exec_()		# main loop
