#!/usr/bin/python3

from PyQt5.QtGui import *
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *

import psycopg2, sys

class MainWindow(QMainWindow):   # Creating main window class
	dbcursor = None
	dbpasswd = None
	subjects = ['Mathematics', 'Statistics', 'Physics', 'Philosophy', 'Hindi', 'Malayalam']
	degrees = ['BA', 'BSc', 'MA', 'MSc']
	syllYears = ['2020', '2021', '2022', '2023']
	semesters = ['First', 'Second', 'Third', 'Fourth', 'Fifth', 'Sixth', 'Seventh', 'Eighth']
	languages = ['english', 'hindi', 'malayalam']
	qpModels = ['M32', 'M40' , 'M60', 'M80A', 'M80B', 'M48']
	durations = ['2 hours', '3 hours']
	 
	def __init__(self, *args, **kwargs):
		super(MainWindow, self).__init__(*args, **kwargs)
		
		self.setGeometry(100, 100, 1000, 600)			
		fixedfont = QFontDatabase.systemFont(QFontDatabase.FixedFont)
		fixedfont.setPointSize(10)

		layout = QVBoxLayout()
		
		vbox1 = QVBoxLayout()
		layout.addLayout(vbox1)

		hbox = QHBoxLayout()
		vbox1.addLayout(hbox)
		x = QLabel('Server Name ')
		hbox.addWidget(x)
		self.hostLineEdit = QLineEdit('openqp.scischool.in')
		self.hostLineEdit.setMaximumWidth(200)
		hbox.addWidget(self.hostLineEdit)
		x = QLabel('Database Name ')
		hbox.addWidget(x)
		self.dbLineEdit = QLineEdit('kuqbank')
		self.dbLineEdit.setMaximumWidth(200)
		hbox.addWidget(self.dbLineEdit)
		self.listButton = QPushButton('List Courses')
		self.listButton.clicked.connect(self.updateCourseList)
		hbox.addWidget(self.listButton)

		hbox.addStretch()

		x = QLabel('To Add a Course provide all the information. To remove,  name is enough.')
		vbox1.addWidget(x)
		
		hbox1 = QHBoxLayout()
		layout.addLayout(hbox1)

		f = QFormLayout()
		self.codeLineEdit = QLineEdit('')
		f.addRow(QLabel("Course Code"), self.codeLineEdit)

		self.pass1LineEdit = QLineEdit('')
		self.pass1LineEdit.setEchoMode(QLineEdit.Password)
		f.addRow(QLabel("Course password"), self.pass1LineEdit)

		self.pass2LineEdit = QLineEdit('')
		self.pass2LineEdit.setEchoMode(QLineEdit.Password)
		f.addRow(QLabel("Course password"), self.pass2LineEdit)

		self.nameLineEdit = QLineEdit('')
		f.addRow(QLabel("Course Name"), self.nameLineEdit)

		self.subjectLineEdit = QLineEdit('')
		f.addRow(QLabel("Subject"), self.subjectLineEdit)

		self.degreeLineEdit = QLineEdit('')
		f.addRow(QLabel("Degree"), self.degreeLineEdit)

		self.syllYearLineEdit = QLineEdit('')
		f.addRow(QLabel("Syllabus Year"), self.syllYearLineEdit)

		self.semLineEdit = QLineEdit('')
		f.addRow(QLabel("Semester"), self.semLineEdit)

		self.langLineEdit = QLineEdit('')
		f.addRow(QLabel("Type setting Language"), self.langLineEdit)

		self.qpModelLineEdit = QLineEdit('')
		f.addRow(QLabel("Question paper model"), self.qpModelLineEdit)

		self.durationLineEdit = QLineEdit('')
		f.addRow(QLabel("Exam Duration"), self.durationLineEdit)
		vbox1.addLayout(f)
		
		self.addButton = QPushButton('Create')
		self.addButton.clicked.connect(self.addCourse)
		hbox1.addWidget(self.addButton)
		self.delButton = QPushButton('Delete')
		self.delButton.clicked.connect(self.delCourse)
		hbox1.addWidget(self.delButton)
		hbox1.addStretch()

		x = QLabel('List of Courses currently available in the Database')
		layout.addWidget(x)

		self.qeditor = QPlainTextEdit()
		self.qeditor.setFont(fixedfont)
		self.qeditor.setLineWrapMode(1)
		layout.addWidget(self.qeditor)
		
		container = QWidget()
		container.setLayout(layout)
		self.setCentralWidget(container)  	# making it central widget	
		self.status = QStatusBar()
		self.setStatusBar(self.status)
		
		subjectMenu = self.menuBar().addMenu("&Subject")
		for sub in self.subjects:
			subjectMenu.addAction(sub, lambda item=sub: self.selectSubject(item))
					
		degreeMenu = self.menuBar().addMenu("&Degree")
		for sub in self.degrees:
			degreeMenu.addAction(sub, lambda item=sub: self.selectDegree(item))
			
		yearsMenu = self.menuBar().addMenu("&Year")
		for sub in self.syllYears:
			yearsMenu.addAction(sub, lambda item=sub: self.selectYear(item))

		semesterMenu = self.menuBar().addMenu("&Semester")
		for sub in self.semesters:
			semesterMenu.addAction(sub, lambda item=sub: self.selectSemester(item))

		languageMenu = self.menuBar().addMenu("&Language")
		for sub in self.languages:
			languageMenu.addAction(sub, lambda item=sub: self.selectLanguage(item))

		qpModelMenu = self.menuBar().addMenu("&PaperModel")
		for sub in self.qpModels:
			qpModelMenu.addAction(sub, lambda item=sub: self.selectqpModel(item))

		durationMenu = self.menuBar().addMenu("&Exam Duration")
		for sub in self.durations:
			durationMenu.addAction(sub, lambda item=sub: self.selectDuration(item))

		self.db_menu = self.menuBar().addMenu("&Select a Course") #filled later

		self.show()		# End __init__
		
	def selectSubject(self,s):
		self.subjectLineEdit.setText(s)
				
	def selectDegree(self,s):
		self.degreeLineEdit.setText(s)

	def selectYear(self,s):
		self.syllYearLineEdit.setText(s)

	def selectSemester(self,s):
		self.semLineEdit.setText(s)

	def selectLanguage(self,s):
		self.langLineEdit.setText(s)

	def selectqpModel(self,s):
		self.qpModelLineEdit.setText(s)

	def selectDuration(self,s):
		self.durationLineEdit.setText(s)

	def selectSubject(self,s):
		self.subjectLineEdit.setText(s)

	def updateCourseList(self):
		host = self.hostLineEdit.text()
		db = self.dbLineEdit.text()
		try:
			conn = psycopg2.connect(database=db, host=host, user='dummy', password='dum123')
			cursor = conn.cursor()
			sql = 'select * from coursedetails;'
			cursor.execute(sql)
			res = cursor.fetchall()
		except:
			s = 'List Courses: Connection to Database %s failed.'%db
			self.statusBar().showMessage(s)		
			return				

		self.courses = res		# Store the course list information
		txt = ''
		self.qeditor.setPlainText(txt)		
		for row in res:
			s = ''
			for k in row:
				s += '%s | '%(k)
			txt += s[:-1] + '\n'  
		self.qeditor.setPlainText(txt)
		s = '%d Courses in Database %s.'%(len(res),db)
		self.statusBar().showMessage(s)		
		
		self.db_menu.clear()
		for k in self.courses:
			self.db_menu.addAction(k[0], lambda item=k[0]: self.selectRemoteCourse(item))
		
		
	def selectRemoteCourse(self, code):
		for c in self.courses:
			if code == c[0]:	
				self.codeLineEdit.setText(c[0])
				self.pass1LineEdit.setText('')
				self.pass2LineEdit.setText('')
				self.nameLineEdit.setText(c[1])
				self.selectSubject(c[2])	
				self.selectDegree(c[3])
				self.selectYear(c[4])
				self.selectSemester(c[5])
				self.selectLanguage(c[6])
				self.selectqpModel(c[7])
				self.selectDuration(c[8])				
				#print (c)
		
	def addCourse(self):	
		code = self.codeLineEdit.text()
		
		for c in self.courses:
			if code == c[0]:
				self.dialog_critical('Existing Course Code | %s'%code)
				return	

		if code.strip() == '':
			self.dialog_critical('Course Code not given.')
			return	

		cpw = self.pass1LineEdit.text()
		if len(cpw) < 6:
			self.dialog_critical('Course password must have minimum 6 characters.')
			return	

		if cpw != self.pass2LineEdit.text():
			self.dialog_critical('Course passwords not matching.')
			return	
			
		name = self.nameLineEdit.text()
		if len(name.strip()) < 5:
			self.dialog_critical('Course Name too short or not given.')
			return	
			
		subject = self.subjectLineEdit.text()
		if subject.strip() == '':
			self.dialog_critical('Subject not selected.')
			return	

		degree = self.degreeLineEdit.text()
		if degree.strip() == '':
			self.dialog_critical('Degree not selected.')
			return	

		syllabusyear = self.syllYearLineEdit.text()
		if syllabusyear.strip() == '':
			self.dialog_critical(' not selected.')
			return	

		semester = self.semLineEdit.text()	
		if semester.strip() == '':
			self.dialog_critical('Semester not selected.')
			return	

		lang = self.langLineEdit.text()
		if lang.strip() == '':
			self.dialog_critical('Type setting language not selected.')
			return	

		qpmodel = self.qpModelLineEdit.text()
		if qpmodel.strip() == '':
			self.dialog_critical('QP Model not selected.')
			return	

		duration = self.durationLineEdit.text()
		if duration.strip() == '':
			self.dialog_critical('Duration not selected.')
			return	

		host = self.hostLineEdit.text()
		db = self.dbLineEdit.text()

		try:
			pw = GUI.getPassword(db)	
			if pw == None:
				GUI.statusBar().showMessage('No password obtained.')		
				return
			conn = psycopg2.connect(database=db, host=host, user=db, password=pw)
			cursor = conn.cursor()
		except:
			self.dialog_critical('Connecting to Database failed.')
			return	
		
		sql = "CREATE TABLE %s(qtext VARCHAR, atext VARCHAR, author VARCHAR, partnum int, qtype CHAR(1), qnum int, relations VARCHAR, mark int, status VARCHAR,figname VARCHAR, figure bytea);"%code
		#print (sql)
		cursor.execute(sql)

		sql = "create user %s with password '%s'"%(code, cpw)
		print (sql)
		cursor.execute(sql)
		
		sql = 'grant all on table %s to %s'%(code,code)
		cursor.execute(sql)

		sql = "insert into coursedetails (code, name, subject, degree, syllabusyear, semester, texlanguage, qpmodel, duration) values ('%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s');"\
			%(code, name, subject, degree, syllabusyear, semester, lang, qpmodel, duration)
		cursor.execute(sql)
		conn.commit()
		self.updateCourseList()
		
		
	def delCourse(self):
		code = self.codeLineEdit.text()

		host = self.hostLineEdit.text()
		db = self.dbLineEdit.text()

		try:
			pw = GUI.getPassword(db)	
			if pw == None:
				GUI.statusBar().showMessage('No password obtained.')		
				return
			conn = psycopg2.connect(database=db, host=host, user=db, password=pw)
			cursor = conn.cursor()
		except:
			self.dialog_critical('Connecting to Database failed.')
			return	

		try:
			sql = "delete from coursedetails where code = '%s'"%code
			cursor.execute(sql)		
			sql = "drop table %s;"%code
			cursor.execute(sql)		
			sql = "drop user %s;"%code
			cursor.execute(sql)
		except:
			print ('Error: Delete Course')
		conn.commit()
		self.updateCourseList()
				
	def getPassword(self, cname):
		text, ok = QInputDialog.getText(None, "Accessing Database", "Password for Database '%s'"%cname, 
		QLineEdit.Password)
		if ok and text:
			return text		

	def dialog_critical(self, s):
		dlg = QMessageBox(self)
		dlg.setText(s)
		dlg.setIcon(QMessageBox.Critical)
		dlg.show()			
			

app = QApplication(sys.argv)   # Main program
GUI = MainWindow()
app.exec_()		# main loop
